from dataclasses import dataclass
from enum import Enum
import abc
from typing import List, Union


class PredefinedAIArgType(Enum):
    String = 'String'
    Integer = 'Integer'
    Float = 'Float'
    Categorical = 'Categorical'
    MultipleOf2 = 'MultipleOf2'


class PredefinedAIArg:
    @abc.abstractmethod
    def get_key(self):
        raise NotImplementedError

    @abc.abstractmethod
    def get_value(self):
        raise NotImplementedError


@dataclass
class PredefinedAIStringArg(PredefinedAIArg):
    key: str
    displayName: str
    description: str
    value: str

    def get_key(self):
        return self.key

    def get_value(self):
        return self.value


@dataclass
class PredefinedAIBoolArg(PredefinedAIArg):
    key: str
    displayName: str
    description: str
    value: bool

    def get_key(self):
        return self.key

    def get_value(self):
        return self.value


@dataclass
class PredefinedAIIntegerArg(PredefinedAIArg):
    key: str
    displayName: str
    description: str
    min: int
    max: int
    value: int
    unitStep: int
    type: PredefinedAIArgType = PredefinedAIArgType.Integer

    def get_key(self):
        return self.key

    def get_value(self):
        return self.value


@dataclass
class PredefinedAIFloatArg(PredefinedAIArg):
    key: str
    displayName: str
    description: str
    min: float
    max: float
    value: float
    unitStep: float
    type: PredefinedAIArgType = PredefinedAIArgType.Float

    def get_key(self):
        return self.key

    def get_value(self):
        return self.value


@dataclass
class PredefinedAIMultipleOf2Arg(PredefinedAIArg):
    key: str
    displayName: str
    description: str
    min: int
    max: int
    value: int
    unitStep: int
    type: PredefinedAIArgType = PredefinedAIArgType.MultipleOf2

    def get_key(self):
        return self.key

    def get_value(self):
        return self.value


@dataclass
class PredefinedAICategoricalValue:
    key: str
    displayName: str
    description: str


@dataclass
class PredefinedAICategoricalArg(PredefinedAIArg):
    key: str
    displayName: str
    description: str
    values: List[PredefinedAICategoricalValue]
    value: PredefinedAICategoricalValue
    type: PredefinedAIArgType = PredefinedAIArgType.Categorical

    def get_key(self):
        return self.key

    def get_value(self):
        return self.value.key


@dataclass
class HyperParameters:
    algorithmHyperParameters: List[
        Union[PredefinedAIBoolArg, PredefinedAIIntegerArg, PredefinedAIFloatArg, PredefinedAIMultipleOf2Arg, PredefinedAICategoricalArg]]
    modelHyperParameters: List[
        Union[PredefinedAIBoolArg, PredefinedAIIntegerArg, PredefinedAIFloatArg, PredefinedAIMultipleOf2Arg, PredefinedAICategoricalArg]]
    samplingHyperParameters: List[
        Union[PredefinedAIBoolArg, PredefinedAIIntegerArg, PredefinedAIFloatArg, PredefinedAIMultipleOf2Arg, PredefinedAICategoricalArg]]
    metricHyperParameters: List[
        Union[PredefinedAIBoolArg, PredefinedAIIntegerArg, PredefinedAIFloatArg, PredefinedAIMultipleOf2Arg, PredefinedAICategoricalArg]]

    def __post_init__(self):
        self.hyperparameter_dict = {}
        for hp_group in self.__annotations__.keys():
            for hp in self.__getattribute__(hp_group):
                self.hyperparameter_dict[hp.get_key()] = hp.get_value()


@dataclass
class Config:
    args: List[
        Union[
            PredefinedAIStringArg, PredefinedAIBoolArg, PredefinedAIIntegerArg, PredefinedAIFloatArg, PredefinedAIMultipleOf2Arg, PredefinedAICategoricalArg]]


@dataclass
class PredefinedAIModelConfig:
    key: str
    displayName: str
    description: str
    hyperParameters: HyperParameters
    train: Config
    retrain: Config
    inference: Config
    serving: Config

    def _get_hyperparameters_dict(self) -> dict:
        return self.hyperParameters.__getattribute__('hyperparameter_dict')

    def _get_args_dict(self, stage) -> dict:
        args_dict = {}
        args = self.__getattribute__(stage).__getattribute__('args')
        for arg in args:
            args_dict[arg.get_key()] = arg.get_value()
        return args_dict




from dataclasses import dataclass
from enum import Enum
from typing import List, Union
from typing_extensions import TypedDict


class StrEnum(str, Enum):
    def __repr__(self):
        return str(self.name)

    def __str__(self):
        return str(self.name)


class ColumnType(StrEnum):
    Numerical = 'Numerical'
    Categorical = 'Categorical'
    Textual = 'Textual'
    TimeStamp = 'TimeStamp'
    Auto = 'Auto'


@dataclass
class Column:
    name: str = ''
    type: ColumnType = ColumnType.Auto


class ColumnDict(TypedDict):
    name: str
    type: ColumnType


class DataConfigurationType(StrEnum):
    Data = 'Data'
    Column = 'Column'
    Columns = 'Columns'


class DefinedConstraint(StrEnum):
    ColumnNumberConstraint = 'ColumnNumberConstraint'
    ColumnRequiredConstraint = 'ColumnRequiredConstraint'
    RecordNumberConstraint = 'RecordNumberConstraint'


@dataclass
class Constraint:
    dataKey: str
    key: str
    type: DataConfigurationType
    constraint: DefinedConstraint
    value: int


@dataclass
class DataConfiguration:
    displayName: str
    key: str
    type: DataConfigurationType
    value: Union[Column, List[Column]]
    required: bool
    constraints: List[Constraint]
    description: str


@dataclass
class PredefinedAIDataConfig:
    displayName: str
    key: str
    configurations: List[DataConfiguration]
    description: str
    constraints: List[Constraint]




import yaml
from mlplatform_lib.mlplatform_lib.predefinedai.predefinedai_data_config import PredefinedAIDataConfig, DataConfigurationType, ColumnDict, Column
from mlplatform_lib.mlplatform_lib.predefinedai.predefinedai_model_config import PredefinedAIModelConfig
from mlplatform_lib.mlplatform_lib.utils.dataclass_utils import from_dict
import argparse
import json
import sys
from typing import List, Union, Dict


class PredefinedAIConfigParser:
    def __init__(self, data_config_path=None, model_config_path=None):
        parser = argparse.ArgumentParser()
        parser.add_argument("--data_config", type=str, default=None)
        parser.add_argument("--model_config", type=str, default=None)
        _args = parser.parse_args()

        if _args.data_config is not None:
            data_config_dict = json.loads(_args.data_config)
        elif data_config_path is not None and model_config_path is not None:
            with open(data_config_path) as data_config_file:
                data_config_dict = yaml.load(data_config_file, yaml.FullLoader)
        else:
            raise ValueError('data_config must be in args')

        if _args.model_config is not None:
            model_config_dict = json.loads(_args.model_config)
        elif model_config_path is not None:
            with open(model_config_path) as model_config_file:
                model_config_dict = yaml.load(model_config_file, yaml.FullLoader)
        else:
            raise ValueError('model_config must be in args')

        self.data_config_list: List[PredefinedAIDataConfig] = [from_dict(PredefinedAIDataConfig, data_config_d) for data_config_d in data_config_dict]
        self.model_config: PredefinedAIModelConfig = from_dict(PredefinedAIModelConfig, model_config_dict)

    def get_data_config_from_data_key(self, data_key) -> PredefinedAIDataConfig:
        try:
            return list(filter(lambda x: x.__getattribute__('key') == data_key, self.data_config_list))[0]
        except IndexError:
            raise ValueError(f'data_key "{data_key}" do not exist in data_config')

    def get_data_config_dict_from_data_key(self, data_key) -> Dict[str, Union[ColumnDict, List[ColumnDict]]]:
        data_config_dict = {}
        data_config = self.get_data_config_from_data_key(data_key)
        for config in data_config.configurations:
            if config.type == DataConfigurationType.Column:
                if config.value is None:
                    data_config_dict[config.key] = None
                else:
                    data_config_dict[config.key] = {'name': config.value.name, 'type': config.value.type}
            elif config.type == DataConfigurationType.Columns:
                data_config_dict[config.key] = []
                if config.value is None:
                    continue
                for value in config.value:
                    data_config_dict[config.key].append({'name': value.name, 'type': value.type})
        return data_config_dict

    def get_all_data_config_dict(self) -> Dict[str, Dict[str, Union[ColumnDict, List[ColumnDict]]]]:
        all_data_config_dict = {}
        data_keys = [data_config.key for data_config in self.data_config_list]
        for data_key in data_keys:
            all_data_config_dict[data_key] = self.get_data_config_dict_from_data_key(data_key)
        return all_data_config_dict

    def get_train_config_dict(self) -> dict:
        return self._get_model_config_dict('train')

    def get_retrain_config_dict(self) -> dict:
        return self._get_model_config_dict('retrain')

    def get_inference_config_dict(self) -> dict:
        return self._get_model_config_dict('inference')

    def get_serving_config_dict(self) -> dict:
        return self._get_model_config_dict('serving')

    def _get_model_args_dict(self, stage) -> dict:
        return self.model_config._get_args_dict(stage)

    def _get_model_config_dict(self, stage) -> dict:
        model_config_dict = {}
        hp_dict = self.model_config._get_hyperparameters_dict()
        args_dict = self.model_config._get_args_dict(stage)
        model_config_dict.update(hp_dict)
        model_config_dict.update(args_dict)
        return model_config_dict
